@@ -1,14 +1,15 @@
-__version__ = '0.7.3'
+__version__ = '0.8.0'
 
-from ._mpservice import ModelService, Modelet
+from ._mpservice import Server, Servlet
 from ._mpservice import TimeoutError, EnqueueTimeout, TotalTimeout
 from ._mperror import MPError
 from ._server_process import ServerProcess
-
+from ._http import run_app
 
 __all__ = [
-    'Modelet', 'ModelService',
+    'Servlet', 'Server',
     'ServerProcess',
     'MPError',
     'TimeoutError', 'EnqueueTimeout', 'TotalTimeout',
+    'run_app',
 ]
