@@ -1,9 +1,4 @@
 
-'''
-Although named after machine learning use cases,
-this service utility is generic.
-'''
-
 import asyncio
 import logging
 import multiprocessing as mp
@@ -32,9 +27,9 @@ class TotalTimeout(TimeoutError):
     pass
 
 
-class Modelet(metaclass=ABCMeta):
+class Servlet(metaclass=ABCMeta):
     # Typically a subclass needs to enhance
-    # `__init__` and implement `predict`.
+    # `__init__` and implement `process`.
 
     @classmethod
     def run(cls, *,
@@ -46,11 +41,11 @@ class Modelet(metaclass=ABCMeta):
             **init_kwargs):
         if cpus:
             psutil.Process().cpu_affinity(cpus=cpus)
-        modelet = cls(**init_kwargs)
-        modelet.start(q_in=q_in,
-                      q_out=q_out,
-                      q_err=q_err,
-                      q_in_lock=q_in_lock)
+        obj = cls(**init_kwargs)
+        obj.start(q_in=q_in,
+                  q_out=q_out,
+                  q_err=q_err,
+                  q_in_lock=q_in_lock)
 
     def __init__(self, *,
                  batch_size: int = None,
@@ -69,9 +64,9 @@ class Modelet(metaclass=ABCMeta):
             uid, x = q_in.get()
             try:
                 if batch_size:
-                    y = self.predict([x])[0]
+                    y = self.process([x])[0]
                 else:
-                    y = self.predict(x)
+                    y = self.process(x)
                 q_out.put((uid, y))
 
             except Exception as e:
@@ -126,7 +121,7 @@ class Modelet(metaclass=ABCMeta):
                             n_batches, batch_size_max, batch_size_min, batch_size_mean)
 
             try:
-                results = self.predict(batch)
+                results = self.process(batch)
             except Exception as e:
                 if not silent_errors or not isinstance(e, silent_errors):
                     logger.info(e)
@@ -138,6 +133,7 @@ class Modelet(metaclass=ABCMeta):
                     q_out.put((uid, y))
 
     def start(self, *, q_in, q_out, q_err, q_in_lock):
+        q_err.put('ready')
         logger.info('%s started', self.name)
         if self.batch_size > 1:
             self._start_batch(q_in=q_in, q_out=q_out, q_err=q_err,
@@ -146,7 +142,7 @@ class Modelet(metaclass=ABCMeta):
             self._start_single(q_in=q_in, q_out=q_out, q_err=q_err)
 
     @abstractmethod
-    def predict(self, x):
+    def process(self, x):
         # `x`: a single element if `self.batch_size == 0`;
         # else, a list of elements.
         # When `batch_size == 0`, hence `x` is a single element,
@@ -156,7 +152,7 @@ class Modelet(metaclass=ABCMeta):
         raise NotImplementedError
 
 
-class ModelService:
+class Server:
     MP_CLASS = mp
     # This class attribute is provided because in some cases
     # on may want to use `torch.multiprocessing`, which is
@@ -175,7 +171,7 @@ class ModelService:
 
         self._uid_to_futures: Dict[int, asyncio.Future] = {}
         self._t_gather_results: asyncio.Task = None  # type: ignore
-        self._modelets: List[mp.Process] = []
+        self._servlets: List[mp.Process] = []
 
         self.loop = asyncio.get_running_loop()
 
@@ -184,13 +180,13 @@ class ModelService:
 
         self._started = False
 
-    def add_modelet(self,
-                    modelet: Type[Modelet],
+    def add_servlet(self,
+                    servlet: Type[Servlet],
                     *,
                     cpus=None,
                     workers: int = None,
                     **init_kwargs):
-        # `modelet` is the class object, not instance.
+        # `servlet` is the class object, not instance.
         assert not self._started
         q_in = self._q_in_out[-1]
         q_in_lock = self.MP_CLASS.Lock()
@@ -223,18 +219,18 @@ class ModelService:
 
         for cpu in cpus:
             if cpu is None:
-                logger.info('adding modelet %s', modelet.__name__)
+                logger.info('adding servlet %s', servlet.__name__)
             else:
                 if isinstance(cpu, int):
                     cpu = [cpu]
                 assert all(0 <= c < n_cpus for c in cpu)
-                logger.info('adding modelet %s at CPU %s',
-                            modelet.__name__, cpu)
+                logger.info('adding servlet %s at CPU %s',
+                            servlet.__name__, cpu)
 
-            self._modelets.append(
+            self._servlets.append(
                 self.MP_CLASS.Process(
-                    target=modelet.run,
-                    name=f'modelet-{cpu}',
+                    target=servlet.run,
+                    name=f'servlet-{cpu}',
                     kwargs={
                         'q_in': q_in,
                         'q_out': q_out,
@@ -247,10 +243,19 @@ class ModelService:
             )
 
     def start(self):
-        assert self._modelets
+        assert self._servlets
         assert not self._started
-        for m in self._modelets:
+        n = 0
+        for m in self._servlets:
             m.start()
+            n += 1
+        k = 0
+        while k < n:
+            z = self._q_err.get()
+            assert z == 'ready'
+            k += 1
+            print(f"servlet processes ready: {k}/{n}")
+
         self._t_gather_results = asyncio.create_task(self._gather_results())
         self._started = True
 
@@ -260,7 +265,7 @@ class ModelService:
         if self._t_gather_results is not None and not self._t_gather_results.done():
             self._t_gather_results.cancel()
             # self._t_gather_results = None
-        for m in self._modelets:
+        for m in self._servlets:
             # if m.is_alive():
             m.terminate()
             m.join()
@@ -293,22 +298,25 @@ class ModelService:
                 uid, err = q_err.get_nowait()
                 fut = futures.pop(uid, None)
                 if fut is None:
-                    continue  # timed-out in `a_predict`.
+                    logger.error(
+                        'got error for an already-cancelled task: %r', err)
+                    continue  # timed-out in `__call__`.
                 try:
                     fut.set_exception(err)
                 except asyncio.InvalidStateError:
                     if fut.cancelled():
-                        logger.warning('Future object is already cancelled')
+                        logger.error(
+                            'got error for an already-cancelled task: %r', err)
                 # No sleep. Get results out of the queue as quickly as possible.
 
             await asyncio.sleep(0.0013)
 
-    async def a_predict(self,
-                        x,
-                        *,
-                        enqueue_timeout: Union[int, float] = None,
-                        total_timeout: Union[int, float] = None,
-                        ):
+    async def __call__(self,
+                       x,
+                       *,
+                       enqueue_timeout: Union[int, float] = None,
+                       total_timeout: Union[int, float] = None,
+                       ):
         if enqueue_timeout is None:
             enqueue_timeout = 10
         elif enqueue_timeout < 0:
@@ -348,7 +356,11 @@ class ModelService:
             await asyncio.wait_for(fut, timeout=time2 - loop.time())
         except asyncio.TimeoutError:
             # `fut` is now cancelled.
-            del self._uid_to_futures[uid]
+            if uid in self._uid_to_futures:
+                # `uid` could have been deleted by
+                # `_gather_results` during very subtle
+                # timing coincidence.
+                del self._uid_to_futures[uid]
             raise TotalTimeout(f'waited {loop.time() - time0} seconds')
         else:
             return fut.result()
